// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

namespace EonaCat.Network
{
    public enum CloseStatusCode : ushort
    {
        //1000: Normal Closure
        //Indicates a normal closure, meaning that the purpose for which the connection was established has been fulfilled.

        //1001: Going Away
        //The endpoint is going away, such as a server going down or a browser tab is closing.

        //1002: Protocol Error
        //Indicates that the endpoint is terminating the connection due to a protocol error.

        //1003: Unsupported Data
        //Indicates that the endpoint is terminating the connection because it received data that it cannot process.

        //1005: No Status Received
        //Reserved. It indicates that no status code was received.

        //1006: Abnormal Closure
        //Reserved. It is a reserved value and should not be set as a status code in a Close control frame by an endpoint.

        //1007: Invalid frame payload data
        //Indicates that an endpoint is terminating the connection because it has received data within a message that was not consistent with the type of the message.

        //1008: Policy Violation
        //Indicates that an endpoint is terminating the connection because it has received a message that violates its policy.

        //1009: Message Too Big
        //Indicates that an endpoint is terminating the connection because it has received a message that is too big for it to process.

        //1010: Missing Extension
        //Indicates that an endpoint is terminating the connection because it has received a message that requires negotiation of an extension, and the client did not offer that extension.

        //1011: Internal Error
        //Indicates that a server is terminating the connection because it encountered an unexpected condition that prevented it from fulfilling the request.

        //1012: Service Restart
        //Reserved. It indicates that the service is restarting.

        //1013: Try Again Later
        //Reserved. It indicates that the server is temporarily unable to process the request.

        //1014: Bad Gateway
        //Reserved. It indicates that an intermediate server failed to fulfill a request.

        //1015: TLS Handshake Failure
        //Reserved. It indicates that the connection was closed due to a failure to perform a TLS handshake.

        Normal = 1000,

        Away = 1001,

        ProtocolError = 1002,

        UnsupportedData = 1003,

        Undefined = 1004,

        NoStatus = 1005,

        Abnormal = 1006,

        InvalidData = 1007,

        PolicyViolation = 1008,

        TooBig = 1009,

        MissingExtension = 1010,

        ServerError = 1011,

        ServiceRestart = 1012,

        TryAgainLater = 1013,

        BadGateway = 1014,

        TlsHandshakeFailure = 1015
    }
}