// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

using System;

namespace EonaCat.Network
{
    /// <summary>
    /// Represents a chunk of data in the EonaCat network library.
    /// </summary>
    internal class WebChunk
    {
        private readonly byte[] _data;
        private int _offset;

        /// <summary>
        /// Initializes a new instance of the <see cref="WebChunk"/> class with the specified data.
        /// </summary>
        /// <param name="data">The byte array representing the data.</param>
        public WebChunk(byte[] data)
        {
            _data = data;
        }

        /// <summary>
        /// Gets the remaining bytes to read in the chunk.
        /// </summary>
        public int ReadLeft => _data.Length - _offset;

        /// <summary>
        /// Reads a specified number of bytes from the chunk into a buffer.
        /// </summary>
        /// <param name="buffer">The destination buffer.</param>
        /// <param name="offset">The zero-based byte offset in the buffer at which to begin storing the data.</param>
        /// <param name="count">The maximum number of bytes to read.</param>
        /// <returns>The actual number of bytes read.</returns>
        public int Read(byte[] buffer, int offset, int count)
        {
            var left = _data.Length - _offset;
            if (left == 0)
            {
                return left;
            }

            if (count > left)
            {
                count = left;
            }

            Buffer.BlockCopy(_data, _offset, buffer, offset, count);
            _offset += count;

            return count;
        }
    }
}