// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

using System;
using System.Runtime.Serialization;
using System.Security.Permissions;

namespace EonaCat.Network
{
    /// <summary>
    /// Represents an exception specific to EonaCat Network cookies.
    /// </summary>
    [Serializable]
    public class CookieException : FormatException, ISerializable
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="CookieException"/> class.
        /// </summary>
        public CookieException()
          : base()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CookieException"/> class with a specified error message.
        /// </summary>
        /// <param name="message">The error message that explains the reason for the exception.</param>
        internal CookieException(string message)
          : base($"EonaCat Network: {message}")
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CookieException"/> class with a specified error message
        /// and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">The error message that explains the reason for the exception.</param>
        /// <param name="innerException">The exception that is the cause of the current exception.</param>
        internal CookieException(string message, Exception innerException)
          : base($"EonaCat Network: {message}", innerException)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CookieException"/> class.
        /// </summary>
        protected CookieException(
          SerializationInfo serializationInfo, StreamingContext streamingContext)
          : base(serializationInfo, streamingContext)
        {
        }

        /// <summary>
        /// Populates a <see cref="SerializationInfo"/> with the data needed to serialize the exception.
        /// </summary>
        /// <param name="serializationInfo">The <see cref="SerializationInfo"/> to populate with data.</param>
        /// <param name="streamingContext">The destination (see <see cref="StreamingContext"/>) for this serialization.</param>
        [SecurityPermission(
          SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]
        public override void GetObjectData(
          SerializationInfo serializationInfo, StreamingContext streamingContext)
        {
            base.GetObjectData(serializationInfo, streamingContext);
        }

        /// <summary>
        /// Populates a <see cref="SerializationInfo"/> with the data needed to serialize the exception.
        /// </summary>
        /// <param name="serializationInfo">The <see cref="SerializationInfo"/> to populate with data.</param>
        /// <param name="streamingContext">The destination (see <see cref="StreamingContext"/>) for this serialization.</param>
        [SecurityPermission(
          SecurityAction.LinkDemand,
          Flags = SecurityPermissionFlag.SerializationFormatter,
          SerializationFormatter = true)]
        void ISerializable.GetObjectData(
          SerializationInfo serializationInfo, StreamingContext streamingContext)
        {
            base.GetObjectData(serializationInfo, streamingContext);
        }
    }
}