// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

using System.Security.Principal;

namespace EonaCat.Network
{
    /// <summary>
    /// Represents a basic HTTP identity with a username and password.
    /// </summary>
    public class HttpBasicIdentity : GenericIdentity
    {
        private readonly string _password;

        /// <summary>
        /// Initializes a new instance of the <see cref="HttpBasicIdentity"/> class.
        /// </summary>
        /// <param name="username">The username associated with the identity.</param>
        /// <param name="password">The password associated with the identity.</param>
        internal HttpBasicIdentity(string username, string password)
          : base(username, "Basic")
        {
            _password = password;
        }

        /// <summary>
        /// Gets the password associated with the identity.
        /// </summary>
        public virtual string Password => _password;
    }
}