// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

using System;

namespace EonaCat.Network
{
    /// <summary>
    /// Represents the type of an HTTP header.
    /// </summary>
    [Flags]
    internal enum HttpHeaderType
    {
        /// <summary>
        /// No specific header type specified.
        /// </summary>
        Unspecified = 0,

        /// <summary>
        /// Header used in an HTTP request.
        /// </summary>
        Request = 1,

        /// <summary>
        /// Header used in an HTTP response.
        /// </summary>
        Response = 1 << 1,

        /// <summary>
        /// Header with restricted use.
        /// </summary>
        Restricted = 1 << 2,

        /// <summary>
        /// Header that can have multiple values.
        /// </summary>
        MultiValue = 1 << 3,

        /// <summary>
        /// Multi-value header used in an HTTP request.
        /// </summary>
        MultiValueInRequest = 1 << 4,

        /// <summary>
        /// Multi-value header used in an HTTP response.
        /// </summary>
        MultiValueInResponse = 1 << 5
    }
}