// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

using System;
using System.ComponentModel;
using System.Runtime.Serialization;

namespace EonaCat.Network
{
    /// <summary>
    /// Represents an exception that occurs in the <see cref="HttpListener"/> class.
    /// </summary>
    [Serializable]
    public class HttpListenerException : Win32Exception
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="HttpListenerException"/> class with no error message.
        /// </summary>
        public HttpListenerException()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="HttpListenerException"/> class with the specified error code.
        /// </summary>
        /// <param name="errorCode">The Win32 error code associated with this exception.</param>
        public HttpListenerException(int errorCode)
          : base(errorCode)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="HttpListenerException"/> class with the specified error code and message.
        /// </summary>
        /// <param name="errorCode">The Win32 error code associated with this exception.</param>
        /// <param name="message">The error message that explains the reason for the exception.</param>
        public HttpListenerException(int errorCode, string message)
          : base(errorCode, $"EonaCat Network: {message}")
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="HttpListenerException"/> class.
        /// </summary>
        protected HttpListenerException(
          SerializationInfo serializationInfo, StreamingContext streamingContext)
          : base(serializationInfo, streamingContext)
        {
        }

        /// <summary>
        /// Gets the Win32 error code associated with this exception.
        /// </summary>
        public override int ErrorCode => NativeErrorCode;
    }
}