// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

using System;

namespace EonaCat.Network
{
    public abstract class WSEndpointHost
    {
        protected WSEndpointHost(string path)
        {
            Path = path;
            Sessions = new WSSessionManager();
        }

        public abstract Type EndpointType { get; }

        public bool AutoCleanSessions
        {
            get
            {
                return Sessions.KeepClean;
            }

            set
            {
                Sessions.KeepClean = value;
            }
        }

        public string Path { get; }
        public WSSessionManager Sessions { get; }

        public TimeSpan ResponseWaitingTime
        {
            get
            {
                return Sessions.ResponseWaitingTime;
            }

            set
            {
                Sessions.ResponseWaitingTime = value;
            }
        }

        internal ServerState State => Sessions.State;

        internal void Start()
        {
            Sessions.Start();
        }

        internal void StartSession(WSContext context)
        {
            CreateSession().Start(context, Sessions);
        }

        internal void Stop(ushort code, string reason)
        {
            Sessions.Stop(code, reason);
        }

        protected abstract WSEndpoint CreateSession();
    }
}