// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

using System;

namespace EonaCat.Network
{
    internal class WebSocketEndpointHost<TEndpoint> : WSEndpointHost
      where TEndpoint : WSEndpoint
    {
        private readonly Func<TEndpoint> _creator;

        internal WebSocketEndpointHost(
          string path, Func<TEndpoint> creator
        )
          : this(path, creator, null)
        {
        }

        internal WebSocketEndpointHost(
          string path,
          Func<TEndpoint> creator,
          Action<TEndpoint> initializer
        )
          : base(path)
        {
            _creator = createCreator(creator, initializer);
        }

        public override Type EndpointType => typeof(TEndpoint);

        protected override WSEndpoint CreateSession()
        {
            return _creator();
        }

        private Func<TEndpoint> createCreator(
                  Func<TEndpoint> creator, Action<TEndpoint> initializer
        )
        {
            if (initializer == null)
            {
                return creator;
            }

            return () =>
            {
                var ret = creator();
                initializer(ret);

                return ret;
            };
        }
    }
}