﻿using System.Collections.Generic;
using System.Security.Cryptography.X509Certificates;
using System.Text;

namespace EonaCat.Network
{
    public class CertificateInfoHelper
    {
        public static string GetCertificatesInformation(IEnumerable<X509Certificate2> certificates)
        {
            StringBuilder stringBuilder = new StringBuilder();

            foreach (var certificate in certificates)
            {
                if (certificate != null)
                {
                    stringBuilder.AppendLine(GetCertificateInformation(certificate));
                }
            }
            return stringBuilder.ToString();
        }

        public static string GetCertificateInformation(X509Certificate2 certificate)
        {
            StringBuilder stringBuilder = new StringBuilder();

            stringBuilder.AppendLine("Certificate Information:");
            stringBuilder.AppendLine($"Subject: {certificate.Subject}");
            stringBuilder.AppendLine($"Issuer: {certificate.Issuer}");
            stringBuilder.AppendLine($"Serial Number: {certificate.SerialNumber}");
            stringBuilder.AppendLine($"Thumbprint: {certificate.Thumbprint}");
            stringBuilder.AppendLine($"Valid From: {certificate.NotBefore}");
            stringBuilder.AppendLine($"Valid Until: {certificate.NotAfter}");
            stringBuilder.AppendLine($"Has Private Key: {certificate.HasPrivateKey}");
            stringBuilder.AppendLine();

            stringBuilder.AppendLine("Public Key Information:");
            stringBuilder.AppendLine($"Algorithm: {certificate.PublicKey.Key.KeyExchangeAlgorithm}");
            stringBuilder.AppendLine($"Key Size: {certificate.PublicKey.Key.KeySize}");
            stringBuilder.AppendLine();

            stringBuilder.AppendLine("Certificate Extensions:");
            foreach (X509Extension extension in certificate.Extensions)
            {
                stringBuilder.AppendLine($"  {extension.Oid.FriendlyName}: {extension.Format(true)}");
            }

            return stringBuilder.ToString();
        }

        public static string GetSubject(X509Certificate2 certificate)
        {
            return certificate.Subject;
        }

        public static string GetIssuer(X509Certificate2 certificate)
        {
            return certificate.Issuer;
        }

        public static string GetValidityPeriod(X509Certificate2 certificate)
        {
            return $"Valid From: {certificate.NotBefore}, Valid Until: {certificate.NotAfter}";
        }
    }
}