﻿using System;
using System.Collections.Generic;

namespace EonaCat.Network
{
    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.

    /// <summary>
    /// Exception event arguments.
    /// </summary>
    public class ExceptionEventArgs : EventArgs
    {
        /// <summary>
        /// IP address.
        /// </summary>
        public string Ip { get; private set; } = null;

        /// <summary>
        /// Port number.
        /// </summary>
        public int Port { get; private set; } = 0;

        /// <summary>
        /// HTTP method.
        /// </summary>
        public HttpMethod Method { get; private set; } = HttpMethod.GET;

        /// <summary>
        /// URL.
        /// </summary>
        public string Url { get; private set; } = null;

        /// <summary>
        /// Request query.
        /// </summary>
        public Dictionary<string, string> Query { get; private set; } = new Dictionary<string, string>();

        /// <summary>
        /// Request headers.
        /// </summary>
        public Dictionary<string, string> RequestHeaders { get; private set; } = new Dictionary<string, string>();

        /// <summary>
        /// Content length.
        /// </summary>
        public long RequestContentLength { get; private set; } = 0;

        /// <summary>
        /// Response status.
        /// </summary>
        public int StatusCode { get; private set; } = 0;

        /// <summary>
        /// Response headers.
        /// </summary>
        public Dictionary<string, string> ResponseHeaders { get; private set; } = new Dictionary<string, string>();

        /// <summary>
        /// Response content length.
        /// </summary>
        public long? ResponseContentLength { get; private set; } = 0;

        /// <summary>
        /// Exception.
        /// </summary>
        public Exception Exception { get; private set; } = null;

        /// <summary>
        /// JSON string of the Exception.
        /// </summary>
        public string Json
        {
            get
            {
                if (Exception != null)
                {
                    return Exception.ToJson(true);
                }

                return null;
            }
        }

        internal ExceptionEventArgs(HttpContext context, Exception e)
        {
            if (context != null)
            {
                Ip = context.Request.Source.IpAddress;
                Port = context.Request.Source.Port;
                Method = context.Request.Method;
                Url = context.Request.Url.Full;
                Query = context.Request.Query.Elements;
                RequestHeaders = context.Request.Headers;
                RequestContentLength = context.Request.ContentLength;
                StatusCode = context.Response.StatusCode;
                ResponseContentLength = context.Response.ContentLength;
            }

            Exception = e;
        }
    }
}